package sat;

import java.util.Calendar;
import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.geometry.VPos;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.TextAlignment;
import javafx.scene.transform.Affine;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Scale;
import javafx.scene.transform.Transform;
import javafx.stage.Stage;

public class Lab01Sat extends Application {
    
    private Canvas background;
    private Line hourHand;
    private Line minuteHand;
    private Line secondHand;
    private final Group   handsGroup = new Group();
    private Scene scene;
    private final Rotate secRotate = new Rotate();
    private final Rotate minRotate = new Rotate();
    private final Rotate hrsRotate = new Rotate();
    private static final int backgroundSize = 200;
    private static final int initialWidth = 300;
    private static final int initialHeight = 250;
    private final MyTimer timer = new MyTimer();

    private class MyTimer extends AnimationTimer
    {
        @Override
        public void handle(long now) 
        {
            double sec = Calendar.getInstance().get(Calendar.SECOND)/60.0;
            double min = (Calendar.getInstance().get(Calendar.MINUTE)+sec)/60.0;
            double hrs = (Calendar.getInstance().get(Calendar.HOUR)%12+min)/12.0;
            secRotate.setAngle(sec*360);
            minRotate.setAngle(min*360);
            hrsRotate.setAngle(hrs*360);
        }        
    }
        
    private void createBackgroundCanvas(double w, double h)
    {
        background = new Canvas(w, h);

        background.widthProperty().addListener( o -> drawBackground() );
        background.heightProperty().addListener( o -> drawBackground() );
        
        drawBackground();
    }
    
    private void createHands()
    {
        hourHand = new Line(0, 0, 0, -(backgroundSize/2)*0.45);
        hourHand.setStroke(Color.BLACK);
        hourHand.setStrokeWidth(6);
        hourHand.getTransforms().setAll(hrsRotate);
        minuteHand = new Line(0, 0, 0, -(backgroundSize/2)*0.7);
        minuteHand.setStroke(Color.BLACK);
        minuteHand.setStrokeWidth(4);
        minuteHand.getTransforms().setAll(minRotate);
        secondHand = new Line(0, 0, 0, -(backgroundSize/2)*0.8);
        secondHand.setStroke(Color.RED);
        secondHand.setStrokeWidth(2);
        secondHand.getTransforms().setAll(secRotate);
        handsGroup.getChildren().addAll(hourHand, minuteHand, secondHand);        
    }
    
    private void drawBackground()
    {
        GraphicsContext gc = background.getGraphicsContext2D();
        
        Affine oldTransform = gc.getTransform();
        
        gc.setFill( Color.GRAY );
        gc.fillRect(0, 0, background.getWidth(), background.getHeight());
        gc.translate( background.getWidth()/2, background.getHeight()/2);
        gc.scale(background.getWidth()/backgroundSize, background.getHeight()/backgroundSize);
        
        final int clockSize = (int)((backgroundSize/2)*0.9);
        
        gc.setFill(Color.WHITE);
        gc.setStroke(Color.BLACK);
        gc.setLineWidth(1);
        gc.fillOval(-clockSize, -clockSize, clockSize*2, clockSize*2);
        gc.strokeOval(-clockSize, -clockSize, clockSize*2, clockSize*2);
        
        Font font = Font.font("Arial", FontWeight.BOLD, 14);
        gc.setFont( font );
        gc.setFill(Color.BLACK);
        gc.setTextAlign(TextAlignment.CENTER);
        gc.setTextBaseline(VPos.CENTER);

        Affine center = gc.getTransform();
        Affine currentTransform = new Affine();

        final int numberOffset = (int)((backgroundSize/2)*0.8);
        
        for(int i = 1; i <= 12; i++)
        {
        double ugao = i * 30;

                currentTransform.setToTransform(center);
                currentTransform.appendRotation( ugao );
                currentTransform.appendTranslation(0, -numberOffset);
                currentTransform.appendRotation( -ugao );

                gc.setTransform(currentTransform);
                gc.fillText(""+i, 0, 0);
        }
        
        gc.setTransform(oldTransform);
    }
   
    private void updateHandScale() 
    {
        handsGroup.layoutXProperty().set(scene.getWidth()/2);
        handsGroup.layoutYProperty().set(scene.getHeight()/2);
        
        Transform scale = new Scale(scene.getWidth()/backgroundSize, scene.getHeight()/backgroundSize);
        handsGroup.getTransforms().setAll(scale);        
    }
    
    @Override
    public void start(Stage primaryStage) {
        Group root = new Group();
        createBackgroundCanvas(initialWidth, initialHeight);
        
        createHands();
        root.getChildren().addAll(background, handsGroup);
        
        scene = new Scene(root, initialWidth, initialHeight);
        
        background.widthProperty().bind( scene.widthProperty() );
        background.heightProperty().bind( scene.heightProperty() );

        scene.widthProperty().addListener( c -> updateHandScale() );
        scene.heightProperty().addListener( c -> updateHandScale() );

        updateHandScale();
        
        primaryStage.setTitle("Časovnik");
        primaryStage.setScene(scene);
        primaryStage.show();

        timer.start();
    }

    public static void main(String[] args) {
        launch(args);
    }
}
