package main;

import sprite.*;
import java.util.ArrayList;
import java.util.List;
import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.*;
import javafx.stage.Stage;

public class Main extends Application {

    public static final double WINDOW_WIDTH = 1200;
    public static final double WINDOW_HEIGHT = 700;

    private AnimationTimer timer;

    private static Group root;
    private Moon moon;
    private Sun sun;

    private static List<Ball> balls = new ArrayList<>();
    private static List<Shot> shots = new ArrayList<>();

    @Override
    public void start(Stage primaryStage) {
        root = new Group();
        Background background = new Background(WINDOW_WIDTH, WINDOW_HEIGHT);
        sun = new Sun(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2);
        moon = new Moon((3.0 / 4) * WINDOW_WIDTH, WINDOW_HEIGHT / 2);
        balls.add(new Ball());
        root.getChildren().addAll(background, sun, moon, balls.get(0));

        Scene scene = new Scene(root, WINDOW_WIDTH, WINDOW_HEIGHT);
        scene.setOnMouseMoved(sun);
        scene.setOnMouseClicked(k -> {
            Main.makeShot(new Shot(sun));
            sun.setRandomMouthColor();
        });
        primaryStage.setTitle("Borba svetlosti");
        primaryStage.setScene(scene);
        primaryStage.setResizable(false);
        primaryStage.show();
        timer = new AnimationTimer() {
            @Override
            public void handle(long now) {
                update();
            }
        };
        timer.start();
    }

    public void update() {
        if (!balls.isEmpty() && balls.get(0).getBoundsInParent().intersects(moon.getBoundsInParent())) {
            root.getChildren().remove(balls.get(0));
            balls.remove(0);
        }
        if (balls.get(balls.size() - 1).getTranslateY() >= Ball.getRadius() * 2) {
            Ball ball = new Ball();
            balls.add(ball);
            root.getChildren().add(ball);
        }

        balls.forEach(ball -> ball.update());

        for (int i = 0; i < shots.size(); i++) {
            Shot shot = shots.get(i);
            shot.update();
            if (shot.getTranslateX() < 0 || shot.getTranslateX() > WINDOW_WIDTH
                    || shot.getTranslateY() < 0 || shot.getTranslateY() > WINDOW_HEIGHT) {
                root.getChildren().remove(shot);
                shots.remove(shot);
            } else {
                for (int j = 0; j < balls.size(); j++) {
                    if (shot.getBoundsInParent().intersects(balls.get(j).getBoundsInParent())) {
                        crashLogic(shot, j);
                        break;
                    }
                }
            }
        }
    }

    //logika za unistavanje pokretnih kugli - NE MENJATI!
    public void crashLogic(Shot shot, int j) {
        Ball hitBall = balls.get(j);
        Ball prevBall = (j - 1 >= 0) ? balls.get(j - 1) : null;
        Ball nextBall = (j + 1 < balls.size()) ? balls.get(j + 1) : null;
        if (nextBall != null && shot.getColor().equals(nextBall.getColor())) {
            j++;
            hitBall = nextBall;
            nextBall = (j + 1 < balls.size()) ? balls.get(j + 1) : null;
            prevBall = (j - 1 >= 0) ? balls.get(j - 1) : null;
        }
        if (shot.getColor().equals(hitBall.getColor())
                && ((prevBall != null && shot.getColor().equals(prevBall.getColor()))
                || (nextBall != null && shot.getColor().equals(nextBall.getColor())))) {
            root.getChildren().remove(shot);
            int sameColorCnt = 0, k;
            //next
            for (k = j + 1; k < balls.size() && balls.get(k).getColor().equals(shot.getColor()); sameColorCnt++) {
                root.getChildren().remove(balls.get(k));
                balls.remove(k);
            }
            //previous
            for (k = j; k >= 0 && balls.get(k).getColor().equals(shot.getColor()); k--, sameColorCnt++) {
                root.getChildren().remove(balls.get(k));
                balls.remove(k);
            }
            //reverse
            for (int m = k; m >= 0; m--) {
                balls.get(m).reverse(sameColorCnt);
            }
        } else {
            if (nextBall != null) {
                shot.becomeMoving(nextBall);
                balls.add(j + 1, shot);
                for (int k = j + 2; k < balls.size(); k++) {
                    Ball hit = balls.get(k);
                    hit.reverse(1);
                }
            } else {
                shot.becomeMoving(balls.get(balls.size() - 1));
                balls.add(balls.size(), shot);
                shot.reverse(1);
            }
        }
        shots.remove(shot);
    }

    public static void makeShot(Shot shot) {
        shots.add(shot);
        root.getChildren().add(shot);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        launch(args);
    }

}
