/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package sprite;

import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import main.Main;

/**
 *
 * @author km183142m
 */
public class Ball extends Sprite {

    protected static final double BALL_VELOCITY = 1.10;
    protected static final double BALL_RADIUS = 16;
    private static final int BORDER_DIVISION = 8;

    private static final Color[] colors = {Color.RED, Color.BLUE, Color.GREEN, Color.YELLOW};
    protected Color color;

    public enum VelocityState {
        UP, DOWN, LEFT, RIGHT
    }
    protected VelocityState velocityState = VelocityState.DOWN;
    protected double velocity;
    
    protected Circle body;
    protected int division = BORDER_DIVISION;
    protected int cnt = BORDER_DIVISION - 1;

    public Ball() {
        body = new Circle(BALL_RADIUS);
        color = colors[(int) (Math.random() * colors.length)];
        body.setFill(color);
        body.setStroke(Color.WHITE);
        getChildren().addAll(body);
        
        velocity = BALL_VELOCITY;
        setTranslateX(1 / 8.0 * Main.WINDOW_WIDTH);
        setTranslateY(0);
    }

    public Color getColor() {
        return color;
    }

    public VelocityState getVelocityState() {
        return velocityState;
    }

    public void checkUpdate(boolean condition, boolean condVelocity, VelocityState state) {
        if (condition) {
            --cnt;
            if (condVelocity) {
                velocity = -velocity;
            }
            velocityState = state;
        }
    }

    @Override
    public void update() {
        division = (cnt < 5 ? 4 : 8);
        switch (velocityState) {
            case UP:
                setTranslateY(getTranslateY() + velocity);
                checkUpdate(getTranslateY() + velocity < (Main.WINDOW_HEIGHT / division),
                        false, Ball.VelocityState.LEFT);
                break;
            case DOWN:
                setTranslateY(getTranslateY() + velocity);
                checkUpdate(getTranslateY() + velocity > (Main.WINDOW_HEIGHT * (1 - 1.0 / division)),
                        false, Ball.VelocityState.RIGHT);
                break;
            case LEFT:
                setTranslateX(getTranslateX() + velocity);
                checkUpdate(getTranslateX() + velocity < (Main.WINDOW_WIDTH / division),
                        true, Ball.VelocityState.DOWN);
                break;
            case RIGHT:
                setTranslateX(getTranslateX() + velocity);
                checkUpdate(getTranslateX() + velocity > (Main.WINDOW_WIDTH * (1 - 1.0 / division)),
                        true, Ball.VelocityState.UP);
                break;
        }
    }

    public void reverse(int times) {
        for (int i = 0; i < times; i++) {
            double step = BALL_RADIUS * 2;
            division = cnt < 3 ? 4 : 8;
            while (step > 0) {
                step -= Math.abs(velocity);
                switch (velocityState) {
                    case UP:
                        setTranslateY(getTranslateY() - velocity);
                        if (getTranslateY() - velocity > (Main.WINDOW_HEIGHT * (1.0 - 1.0 / division))) {
                            cnt++;
                            velocityState = Ball.VelocityState.RIGHT;
                            velocity = -velocity;
                        }
                        break;
                    case DOWN:
                        setTranslateY(getTranslateY() - velocity);
                        if (cnt < BORDER_DIVISION - 1 && getTranslateY() - velocity < (Main.WINDOW_HEIGHT / division)) {
                            cnt++;
                            velocityState = Ball.VelocityState.LEFT;
                            velocity = -velocity;
                        }
                        break;
                    case LEFT:
                        setTranslateX(getTranslateX() - velocity);
                        if (getTranslateX() - velocity > (Main.WINDOW_WIDTH * (1.0 - 1.0 / division))) {
                            cnt++;
                            velocityState = Ball.VelocityState.UP;
                        }
                        break;
                    case RIGHT:
                        setTranslateX(getTranslateX() - velocity);
                        if (cnt < BORDER_DIVISION - 1 && getTranslateX() - velocity < Main.WINDOW_WIDTH / division) {
                            cnt++;
                            velocityState = Ball.VelocityState.DOWN;
                        }
                        break;
                }
            }
        }
    }

    public static double getRadius() {
        return BALL_RADIUS;
    }

}
